/*   azrael - client for the xmms2 player
 *   Copyright (C) 2005  Costin Chirvasuta
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "az_cmd.h"

void cmd_seek(guint ms)
{
  xmmsc_result_t *res;

  res = xmmsc_playback_seek_ms(connection, ms);
  xmmsc_result_wait(res);
  xmmsc_result_unref(res);
}

void cmd_volume_left(guint vol)
{
  xmmsc_result_t *res;

  res = xmmsc_playback_volume_set(connection, "left", vol);
  xmmsc_result_wait(res);
  xmmsc_result_unref(res);
}

void cmd_volume_right(guint vol)
{
  xmmsc_result_t *res;

  res = xmmsc_playback_volume_set(connection, "right", vol);
  xmmsc_result_wait(res);
  xmmsc_result_unref(res);
}

void _get_volume(const void *key, xmmsc_result_value_type_t type, const void *value, void *userdata)
{
       if (!strcmp(key, "left"))  gui_vol_left_set(XPOINTER_TO_INT(value));
  else if (!strcmp(key, "right")) gui_vol_right_set(XPOINTER_TO_INT(value));
}

void cmd_get_volume()
{
  xmmsc_result_t *res;

  res = xmmsc_playback_volume_get(connection);
  xmmsc_result_wait(res);

  xmmsc_result_dict_foreach(res, _get_volume, NULL);

  xmmsc_result_unref(res);
}

void cmd_gain(guint i, double g)
{
  xmmsc_result_t *res;
  char           str[64], str2[64];

  snprintf(str,  63, "effect.equalizer.gain%d", i);
  snprintf(str2, 63, "%3.1f", g);
  res = xmmsc_configval_set(connection, str, str2);
  xmmsc_result_wait(res);
  xmmsc_result_unref(res);
}

void cmd_play_pos(gint pos)
{
  xmmsc_result_t *res;

  res = xmmsc_playlist_set_next(connection, pos);
  xmmsc_result_wait(res);
  xmmsc_result_unref(res);

  res = xmmsc_playback_tickle(connection);
  xmmsc_result_wait(res);
  xmmsc_result_unref(res);

  res = xmmsc_playback_start(connection);
  xmmsc_result_wait(res);
  xmmsc_result_unref(res);
}

static void go_to_pos(int pos)
{
  xmmsc_result_t *res;

  res = xmmsc_playlist_set_next_rel(connection, pos);
  xmmsc_result_wait(res);
  xmmsc_result_unref(res);

  res = xmmsc_playback_tickle(connection);
  xmmsc_result_wait(res);
  xmmsc_result_unref(res);
}

void cmd_prev_song(GtkWidget *widget, gpointer userdata)
{
  go_to_pos(-1);
}

void cmd_next_song(GtkWidget *widget, gpointer userdata)
{
  go_to_pos(1);
}

void cmd_play(GtkWidget *widget, gpointer userdata)
{
  xmmsc_result_t *res;

  res = xmmsc_playback_start(connection);
  xmmsc_result_wait(res);
  xmmsc_result_unref(res);
}

void cmd_pause(GtkWidget *widget, gpointer userdata)
{
  xmmsc_result_t *res;

  res = xmmsc_playback_pause(connection);
  xmmsc_result_wait(res);
  xmmsc_result_unref(res);
}

void cmd_stop(GtkWidget *widget, gpointer userdata)
{
  xmmsc_result_t *res;

  res = xmmsc_playback_stop(connection);
  xmmsc_result_wait(res);
  xmmsc_result_unref(res);
}

void cmd_playlist_shuffle(guint opt)
{
  xmmsc_result_t *res;
  gchar sopt[2][2] = { "0", "1" };
  gchar *popt;

  switch (opt) {
    case 0:  popt=sopt[0]; break;
    case 1:  popt=sopt[1]; break;
    default: popt=sopt[0];
  }

  res = xmmsc_configval_set(connection, "playlist.medialib_random_mode", popt);
  xmmsc_result_wait(res);
  xmmsc_result_unref(res);
}

void cmd_playlist_repeat(guint opt)
{
  xmmsc_result_t *res;
  gchar sopt[2][2] = { "0", "1" };
  gchar *popt;

  switch (opt) {
    case 0:  popt=sopt[0]; break;
    case 1:  popt=sopt[1]; break;
    default: popt=sopt[0];
  }

  res = xmmsc_configval_set(connection, "playlist.repeat_all", popt);
  xmmsc_result_wait(res);
  xmmsc_result_unref(res);
}

void cmd_playlist_repsong(guint opt)
{
  xmmsc_result_t *res;
  gchar sopt[2][2] = { "0", "1" };
  gchar *popt;

  switch (opt) {
    case 0:  popt=sopt[0]; break;
    case 1:  popt=sopt[1]; break;
    default: popt=sopt[0];
  }

  res = xmmsc_configval_set(connection, "playlist.repeat_one", popt);
  xmmsc_result_wait(res);
  xmmsc_result_unref(res);
}

void cmd_playlist_clear(GtkWidget *widget, gpointer userdata)
{
  xmmsc_result_t *res;

  res = xmmsc_playlist_clear(connection);
  xmmsc_result_wait(res);
  xmmsc_result_unref(res);
}

void cmd_playlist_add_id(guint id)
{
  xmmsc_result_t *res;

  res = xmmsc_playlist_add_id(connection, id);
  xmmsc_result_wait(res);
  xmmsc_result_unref(res);
}

