/*   azrael - client for the xmms2 player
 *   Copyright (C) 2005  Costin Chirvasuta
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "azrael.h"
#include "az_gui.h"
#include "az_lcd.h"
#include "az_cmd.h"


static GtkWidget   *window;
static GtkWidget   *_gui_notebook;
static GtkWidget   *_gui_separator2;
static GtkWidget   *_gui_vbox_equalizer;

static gchar       window_title[256];
static guint       window_title_len;
static GdkColor    color_bg;
static GdkColor    color_text;

static GdkGeometry hints;
static gint        maximized;


void gui_error(gchar *str)
{
  GtkWidget *dialog;

  dialog = gtk_message_dialog_new(GTK_WINDOW(window), GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
                                  GTK_MESSAGE_ERROR, GTK_BUTTONS_OK,
                                  "Azrael error:");
  gtk_message_dialog_format_secondary_text(GTK_MESSAGE_DIALOG(dialog), str);

  gtk_dialog_run(GTK_DIALOG(dialog));
  gtk_widget_destroy(dialog);
}

static void cb_lcd_click(GtkWidget *widget, gpointer data)
{
  static gboolean cd=FALSE;

  cd = !cd;
  lcd_set_countdown(cd);
}

static void cb_shuffle_toggle(GtkWidget *button, gpointer data)
{
  if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(button))) {
    cmd_playlist_shuffle(1);
  } else {
    cmd_playlist_shuffle(0);
  }
}

static void cb_repeat_toggle(GtkWidget *button, gpointer data)
{
  if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(button))) {
    cmd_playlist_repeat(1);
  } else {
    cmd_playlist_repeat(0);
  }
}

static void cb_repsong_toggle(GtkWidget *button, gpointer data)
{
  if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(button))) {
    cmd_playlist_repsong(1);
  } else {
    cmd_playlist_repsong(0);
  }
}

static void cb_small_toggle(GtkWidget *button, gpointer data)
{
  static int         hidden;
  static int         x, y;
  static int         sx, sy;
  static int         _maximized;

  if (hidden) {
    gtk_window_get_size(GTK_WINDOW(window), &sx, &sy);
    gtk_window_set_geometry_hints(GTK_WINDOW(window), window, NULL, 0);
    gtk_window_resize(GTK_WINDOW(window), x, y);
    if (_maximized) gtk_window_maximize(GTK_WINDOW(window));
    gtk_widget_show(_gui_separator2);
    gtk_widget_show(_gui_vbox_equalizer);
    gtk_widget_show(_gui_notebook);
    hidden = 0;
  } else {
    if (!sx) sx=700;
    if (maximized) {
      _maximized = 1;
      gtk_window_unmaximize(GTK_WINDOW(window));
    }
    else _maximized = 0;
    gtk_window_get_size(GTK_WINDOW(window), &x, &y);
    gtk_widget_hide(_gui_notebook);
    gtk_widget_hide(_gui_vbox_equalizer);
    gtk_widget_hide(_gui_separator2);
    gtk_window_set_geometry_hints(GTK_WINDOW(window), window, &hints, GDK_HINT_MAX_SIZE);
    gtk_window_resize(GTK_WINDOW(window), sx, 1);
    hidden = 1;
  }
}


static GtkWidget     *entry_finfo;
static gchar         finfo_stat[256];
static GtkAdjustment *adj_prog;
static gboolean      adj_prog_ignore;

static void cb_prog_changed(GtkAdjustment *adj)
{
  if (adj_prog_ignore) {
    adj_prog_ignore = FALSE;
    return;
  }

  cmd_seek(adj->value*1000);
}

void gui_finfo_set(gchar *str)
{
  gtk_entry_set_text(GTK_ENTRY(entry_finfo), str);
}

void gui_finfo_stat(gchar *str)
{
  strncpy(finfo_stat, str, 256);
}

void gui_finfo_stat_show()
{
  gtk_entry_set_text(GTK_ENTRY(entry_finfo), finfo_stat);
}

void gui_prog_set(gdouble value)
{
  gtk_adjustment_set_value(GTK_ADJUSTMENT(adj_prog), value);
}

void gui_prog_clear()
{
  gtk_adjustment_set_value(GTK_ADJUSTMENT(adj_prog), 0);
  adj_prog->upper = 0;
}

void gui_prog_upper(gdouble upper)
{
  adj_prog->upper = upper;
}

void gui_prog_ignore(gboolean ign)
{
  adj_prog_ignore = ign;
}

static GtkWidget *_gui_controls_create()
{
  GtkWidget *lcd;
  GtkWidget *button_lcd;
  GtkWidget *hbox;
  GtkWidget *button_s, *button_r, *button_1;
  GtkWidget *button_small;
  GtkWidget *image;
  GtkWidget *vbox1, *vbox2;
  GtkWidget *hbox_top;
  GtkWidget *scale_prog;
  GtkWidget *button_prev, *button_play, *button_pause, *button_stop, *button_next;
  GtkWidget *hbox_bottom;
  GtkWidget *vbox;

  lcd = lcd_create();
  lcd_set_total(0);
  lcd_set_countdown(FALSE);
  button_lcd = gtk_button_new();
  g_signal_connect(G_OBJECT(button_lcd), "clicked",
                   G_CALLBACK(cb_lcd_click), NULL);
  gtk_container_add(GTK_CONTAINER(button_lcd), lcd);
  gtk_button_set_relief(GTK_BUTTON(button_lcd), GTK_RELIEF_NONE);

  entry_finfo = gtk_entry_new();
  gtk_entry_set_alignment(GTK_ENTRY(entry_finfo), 0.5);
  gtk_entry_set_editable(GTK_ENTRY(entry_finfo), FALSE);
  gtk_widget_modify_base(GTK_WIDGET(entry_finfo), GTK_STATE_NORMAL, &color_bg);
  gtk_widget_modify_text(GTK_WIDGET(entry_finfo), GTK_STATE_NORMAL, &color_text);

  button_s = gtk_check_button_new_with_label("S");
  g_signal_connect(G_OBJECT(button_s), "toggled",
                   G_CALLBACK(cb_shuffle_toggle), NULL);

  button_r = gtk_check_button_new_with_label("R");
  g_signal_connect(G_OBJECT(button_r), "toggled",
                   G_CALLBACK(cb_repeat_toggle), NULL);

  button_1 = gtk_check_button_new_with_label("1");
  g_signal_connect(G_OBJECT(button_1), "toggled",
                   G_CALLBACK(cb_repsong_toggle), NULL);

  button_small = gtk_button_new();
  g_signal_connect(G_OBJECT(button_small), "clicked",
                   G_CALLBACK(cb_small_toggle), NULL);
  gtk_widget_set_size_request(GTK_WIDGET(button_small), -1, 20);
  
  image = gtk_image_new_from_stock("gtk-refresh", GTK_ICON_SIZE_SMALL_TOOLBAR);
  gtk_container_add(GTK_CONTAINER(button_small), image);
                   
  adj_prog = (GtkAdjustment*) gtk_adjustment_new(0, 0, 0, 5, 20, 0);
  adj_prog_ignore = FALSE;
  g_signal_connect(G_OBJECT(adj_prog), "value_changed",
                   G_CALLBACK(cb_prog_changed), NULL);
  scale_prog = gtk_hscale_new(adj_prog);
  gtk_scale_set_draw_value(GTK_SCALE(scale_prog), FALSE);
  gtk_range_set_update_policy(GTK_RANGE(scale_prog), GTK_UPDATE_DISCONTINUOUS);

  button_prev = gtk_button_new_from_stock("gtk-media-previous");
  g_signal_connect(G_OBJECT(button_prev), "clicked",
                   G_CALLBACK(cmd_prev_song), NULL);
  
  button_play = gtk_button_new_from_stock("gtk-media-play");
  g_signal_connect(G_OBJECT(button_play), "clicked",
                   G_CALLBACK(cmd_play), NULL);
  
  button_pause = gtk_button_new_from_stock("gtk-media-pause");
  g_signal_connect(G_OBJECT(button_pause), "clicked",
                   G_CALLBACK(cmd_pause), NULL);
  
  button_stop = gtk_button_new_from_stock("gtk-media-stop");
  g_signal_connect(G_OBJECT(button_stop), "clicked",
                   G_CALLBACK(cmd_stop), NULL);
  
  button_next = gtk_button_new_from_stock("gtk-media-next");
  g_signal_connect(G_OBJECT(button_next), "clicked",
                   G_CALLBACK(cmd_next_song), NULL);
  
  hbox = gtk_hbox_new(FALSE, 0);
  gtk_box_pack_start(GTK_BOX(hbox), button_small, FALSE, FALSE, 4);
  gtk_box_pack_start(GTK_BOX(hbox), button_s,     FALSE, FALSE, 0);
  gtk_box_pack_start(GTK_BOX(hbox), button_r,     FALSE, FALSE, 0);
  gtk_box_pack_start(GTK_BOX(hbox), button_1,     FALSE, FALSE, 0);

  vbox1 = gtk_vbox_new(FALSE, 0);
  gtk_box_pack_start(GTK_BOX(vbox1), button_lcd, TRUE, FALSE, 0);

  vbox2 = gtk_vbox_new(FALSE, 0);
  gtk_box_pack_start(GTK_BOX(vbox2), entry_finfo, FALSE, FALSE, 4);
  gtk_box_pack_start(GTK_BOX(vbox2), hbox,        FALSE, FALSE, 0);

  hbox_top = gtk_hbox_new(FALSE, 0);
  gtk_box_pack_start(GTK_BOX(hbox_top), vbox1, FALSE, FALSE, 0);
  gtk_box_pack_start(GTK_BOX(hbox_top), vbox2, TRUE,  TRUE,  4);

  hbox_bottom = gtk_hbox_new(FALSE, 0);
  gtk_box_pack_start(GTK_BOX(hbox_bottom), button_prev,  FALSE, FALSE, 0);
  gtk_box_pack_start(GTK_BOX(hbox_bottom), button_play,  FALSE, FALSE, 0);
  gtk_box_pack_start(GTK_BOX(hbox_bottom), button_pause, FALSE, FALSE, 0);
  gtk_box_pack_start(GTK_BOX(hbox_bottom), button_stop,  FALSE, FALSE, 0);
  gtk_box_pack_start(GTK_BOX(hbox_bottom), button_next,  FALSE, FALSE, 0);

  vbox = gtk_vbox_new(FALSE, 6);
  gtk_box_pack_start(GTK_BOX(vbox), hbox_top,    FALSE, FALSE, 0);
  gtk_box_pack_start(GTK_BOX(vbox), scale_prog,  FALSE, FALSE, 0);
  gtk_box_pack_start(GTK_BOX(vbox), hbox_bottom, FALSE, FALSE, 0);

  return vbox;
}

static GtkWidget     *scale_vol_left;
static GtkWidget     *scale_vol_right;
static GtkAdjustment *adj_vol;
static GtkAdjustment *adj_vol_left;
static GtkAdjustment *adj_vol_right;
static GtkWidget     *volume_button;
static GtkWidget     *volume_image;
static gint          volume_lock;

static void cb_vol_changed(GtkAdjustment *adj)
{
  gchar str[64];

  snprintf(str, 64, "Volume: %d%%", (int) adj->value);
  gui_finfo_set(str);
  az_set_ticks(2);
  cmd_volume_left((guint)adj->value);
  cmd_volume_right((guint)adj->value);
}

static void cb_vol_left_changed(GtkAdjustment *adj)
{
  gchar str[64];

  snprintf(str, 64, "Volume left: %d%%", (int)adj->value);
  gui_finfo_set(str);
  az_set_ticks(2);
  cmd_volume_left((guint)adj->value);
}

static void cb_vol_right_changed(GtkAdjustment *adj)
{
  gchar str[64];

  snprintf(str, 64, "Volume right: %d%%", (int)adj->value);
  gui_finfo_set(str);
  az_set_ticks(2);
  cmd_volume_right((guint)adj->value);
}

void gui_vol_left_set(guint vol)
{
  gtk_adjustment_set_value(GTK_ADJUSTMENT(adj_vol_left), (gdouble)vol);
}

void gui_vol_right_set(guint vol)
{
  gtk_adjustment_set_value(GTK_ADJUSTMENT(adj_vol_right), (gdouble)vol);
}

static void cb_volume_lock(GtkWidget *widget, gpointer userdata)
{
  volume_lock^=1;

  gtk_widget_destroy(GTK_WIDGET(volume_image));

  if (volume_lock) {
    volume_image = gtk_image_new_from_stock("gtk-connect", GTK_ICON_SIZE_SMALL_TOOLBAR);
    gtk_container_add(GTK_CONTAINER(volume_button), volume_image);
    gtk_widget_show_all(volume_button);

    adj_vol->value = (adj_vol_left->value+adj_vol_right->value)/2;
    gtk_range_set_adjustment(GTK_RANGE(scale_vol_left), adj_vol);
    gtk_range_set_adjustment(GTK_RANGE(scale_vol_right), adj_vol);
  }
  else {
    volume_image = gtk_image_new_from_stock("gtk-disconnect", GTK_ICON_SIZE_SMALL_TOOLBAR);
    gtk_container_add(GTK_CONTAINER(volume_button), volume_image);
    gtk_widget_show_all(volume_button);

    adj_vol_left->value  = adj_vol->value;
    adj_vol_right->value = adj_vol->value;
    gtk_range_set_adjustment(GTK_RANGE(scale_vol_left),  adj_vol_left);
    gtk_range_set_adjustment(GTK_RANGE(scale_vol_right), adj_vol_right);
  }
}

static GtkWidget *_gui_volume_create()
{
  GtkWidget *label;
  GtkWidget *hbox;
  GtkWidget *vbox;

  adj_vol = (GtkAdjustment*)gtk_adjustment_new(0, 0, 100, 2, 5, 0);
  g_signal_connect(G_OBJECT(adj_vol), "value_changed",
                   G_CALLBACK(cb_vol_changed), NULL);

  adj_vol_left  = (GtkAdjustment*)gtk_adjustment_new(0, 0, 100, 2, 5, 0);
  g_signal_connect(G_OBJECT(adj_vol_left), "value_changed",
                   G_CALLBACK(cb_vol_left_changed), NULL);

  adj_vol_right = (GtkAdjustment*)gtk_adjustment_new(0, 0, 100, 2, 5, 0);
  g_signal_connect(G_OBJECT(adj_vol_right), "value_changed",
                   G_CALLBACK(cb_vol_right_changed), NULL);

  g_object_ref(adj_vol_left);
  g_object_ref(adj_vol_right);
  g_object_ref(adj_vol);

  scale_vol_left = gtk_vscale_new(NULL);
  gtk_scale_set_draw_value(GTK_SCALE(scale_vol_left), FALSE);
  gtk_range_set_inverted(GTK_RANGE(scale_vol_left), TRUE);

  scale_vol_right = gtk_vscale_new(NULL);
  gtk_scale_set_draw_value(GTK_SCALE(scale_vol_right), FALSE);
  gtk_range_set_inverted(GTK_RANGE(scale_vol_right), TRUE);

  volume_button = gtk_button_new();
  gtk_widget_set_size_request(GTK_WIDGET(volume_button), -1, 20);
  gtk_button_set_relief(GTK_BUTTON(volume_button), GTK_RELIEF_NONE);
  g_signal_connect(G_OBJECT(volume_button), "clicked",
                   G_CALLBACK(cb_volume_lock), NULL);

  cmd_get_volume();

  if (adj_vol_left->value == adj_vol_right->value) {
    volume_lock = 1;
    
    volume_image = gtk_image_new_from_stock("gtk-connect", GTK_ICON_SIZE_SMALL_TOOLBAR);
    gtk_container_add(GTK_CONTAINER(volume_button), volume_image);

    adj_vol->value = adj_vol_left->value;
    gtk_range_set_adjustment(GTK_RANGE(scale_vol_left), adj_vol);
    gtk_range_set_adjustment(GTK_RANGE(scale_vol_right), adj_vol);
  }
  else {
    volume_lock = 0;
    
    volume_image = gtk_image_new_from_stock("gtk-disconnect", GTK_ICON_SIZE_SMALL_TOOLBAR);
    gtk_container_add(GTK_CONTAINER(volume_button), volume_image);

    gtk_range_set_adjustment(GTK_RANGE(scale_vol_left), adj_vol_left);
    gtk_range_set_adjustment(GTK_RANGE(scale_vol_right), adj_vol_right);
  }

  hbox = gtk_hbox_new(FALSE, 0);
  gtk_box_pack_start(GTK_BOX(hbox), scale_vol_left,  FALSE, FALSE, 0);
  gtk_box_pack_start(GTK_BOX(hbox), scale_vol_right, FALSE, FALSE, 0);

  label = gtk_label_new("");
  
  vbox = gtk_vbox_new(FALSE, 0);
  gtk_box_pack_start(GTK_BOX(vbox), label,         FALSE, FALSE, 0);
  gtk_box_pack_start(GTK_BOX(vbox), hbox,          TRUE,  TRUE,  0);
  gtk_box_pack_start(GTK_BOX(vbox), volume_button, FALSE, FALSE, 0);

  return vbox;
}


static gchar         eq_label[10][10] = { "0", "1", "2", "3", "4", "5", "6", "7", "8", "9" };
//                                      { "60", "170", "310", "600", "1k", "3k", "6k", "12k", "14k", "16k" };
static GtkAdjustment *adj_eq[10];
static gboolean      eq_on = FALSE;

static void cb_eq_toggle(GtkWidget *button, gpointer data)
{
  gint i;

  if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(button))) {
    eq_on = TRUE;
    for (i=0; i<10; i++) cmd_gain(i, adj_eq[i]->value);
  } else {
    eq_on = FALSE;
    for (i=0; i<10; i++) cmd_gain(i, 0.5);
  }
}

static void cb_eq_reset(GtkWidget *button, gpointer data)
{
  gint i;

  for (i=0; i<10; i++) gtk_adjustment_set_value(adj_eq[i], 0.5);
}

static void cb_eq_changed0(GtkAdjustment *adj)
{
  if (!eq_on) return;
  cmd_gain(0, adj->value);
}

static void cb_eq_changed1(GtkAdjustment *adj)
{
  if (!eq_on) return;
  cmd_gain(1, adj->value);
}

static void cb_eq_changed2(GtkAdjustment *adj)
{
  if (!eq_on) return;
  cmd_gain(2, adj->value);
}

static void cb_eq_changed3(GtkAdjustment *adj)
{
  if (!eq_on) return;
  cmd_gain(3, adj->value);
}

static void cb_eq_changed4(GtkAdjustment *adj)
{
  if (!eq_on) return;
  cmd_gain(4, adj->value);
}

static void cb_eq_changed5(GtkAdjustment *adj)
{
  if (!eq_on) return;
  cmd_gain(5, adj->value);
}

static void cb_eq_changed6(GtkAdjustment *adj)
{
  if (!eq_on) return;
  cmd_gain(6, adj->value);
}

static void cb_eq_changed7(GtkAdjustment *adj)
{
  if (!eq_on) return;
  cmd_gain(7, adj->value);
}

static void cb_eq_changed8(GtkAdjustment *adj)
{
  if (!eq_on) return;
  cmd_gain(8, adj->value);
}

static void cb_eq_changed9(GtkAdjustment *adj)
{
  if (!eq_on) return;
  cmd_gain(9, adj->value);
}

static GtkWidget *_gui_equalizer_create()
{
  GtkWidget *button_eq;
  GtkWidget *button_reset;
  GtkWidget *image;
  GtkWidget *scale;
  GtkWidget *label;
  GtkWidget *vbox_scale;
  GtkWidget *hbox_top;
  GtkWidget *hbox_bottom;
  GtkWidget *vbox;
  guint     i;

  button_eq = gtk_check_button_new_with_label("Equalizer");
  g_signal_connect(G_OBJECT(button_eq), "toggled",
                   G_CALLBACK(cb_eq_toggle), NULL);

  button_reset = gtk_button_new();
  image = gtk_image_new_from_stock("gtk-remove", GTK_ICON_SIZE_SMALL_TOOLBAR);
  gtk_container_add(GTK_CONTAINER(button_reset), image);
  gtk_widget_set_size_request(GTK_WIDGET(button_reset), -1, 20);
  g_signal_connect(G_OBJECT(button_reset), "clicked",
                   G_CALLBACK(cb_eq_reset), NULL);

  hbox_bottom = gtk_hbox_new(FALSE, 0);

  for (i=0; i<10; i++) {
    adj_eq[i] = (GtkAdjustment*) gtk_adjustment_new(0.5, 0, 1, 0.05, 0.1, 0);

    switch (i) {
      case 0: g_signal_connect(G_OBJECT(adj_eq[i]), "value_changed", G_CALLBACK(cb_eq_changed0), NULL); break;
      case 1: g_signal_connect(G_OBJECT(adj_eq[i]), "value_changed", G_CALLBACK(cb_eq_changed1), NULL); break;
      case 2: g_signal_connect(G_OBJECT(adj_eq[i]), "value_changed", G_CALLBACK(cb_eq_changed2), NULL); break;
      case 3: g_signal_connect(G_OBJECT(adj_eq[i]), "value_changed", G_CALLBACK(cb_eq_changed3), NULL); break;
      case 4: g_signal_connect(G_OBJECT(adj_eq[i]), "value_changed", G_CALLBACK(cb_eq_changed4), NULL); break;
      case 5: g_signal_connect(G_OBJECT(adj_eq[i]), "value_changed", G_CALLBACK(cb_eq_changed5), NULL); break;
      case 6: g_signal_connect(G_OBJECT(adj_eq[i]), "value_changed", G_CALLBACK(cb_eq_changed6), NULL); break;
      case 7: g_signal_connect(G_OBJECT(adj_eq[i]), "value_changed", G_CALLBACK(cb_eq_changed7), NULL); break;
      case 8: g_signal_connect(G_OBJECT(adj_eq[i]), "value_changed", G_CALLBACK(cb_eq_changed8), NULL); break;
      case 9: g_signal_connect(G_OBJECT(adj_eq[i]), "value_changed", G_CALLBACK(cb_eq_changed9), NULL); break;
    }

    scale = gtk_vscale_new(adj_eq[i]);
    gtk_scale_set_draw_value(GTK_SCALE(scale), FALSE);
    gtk_range_set_update_policy(GTK_RANGE(scale), GTK_UPDATE_DISCONTINUOUS);
    gtk_range_set_inverted(GTK_RANGE(scale), TRUE);

    label = gtk_label_new(eq_label[i]);

    vbox_scale = gtk_vbox_new(FALSE, 0);
    gtk_box_pack_start(GTK_BOX(vbox_scale), scale, TRUE,  TRUE,  0);
    gtk_box_pack_start(GTK_BOX(vbox_scale), label, FALSE, FALSE, 0);

    gtk_box_pack_start(GTK_BOX(hbox_bottom), vbox_scale, FALSE, FALSE, 0);
  }

  hbox_top = gtk_hbox_new(FALSE, 0);
  gtk_box_pack_start(GTK_BOX(hbox_top), button_eq,    FALSE, FALSE, 0);
  gtk_box_pack_end  (GTK_BOX(hbox_top), button_reset, FALSE, FALSE, 0);

  vbox = gtk_vbox_new(FALSE, 0);
  gtk_box_pack_start(GTK_BOX(vbox), hbox_top,    FALSE, FALSE, 0);
  gtk_box_pack_start(GTK_BOX(vbox), hbox_bottom, TRUE,  TRUE,  0);

  return vbox;
}


static GtkTextBuffer *tbuffer_minfo;

void gui_minfo_clear()
{
  GtkTextIter start, end;

  gtk_text_buffer_get_bounds(GTK_TEXT_BUFFER(tbuffer_minfo), &start, &end);
  gtk_text_buffer_delete(GTK_TEXT_BUFFER(tbuffer_minfo), &start, &end);
}

void gui_minfo(char *name, char *value)
{
  GtkTextIter end;

  gtk_text_buffer_get_end_iter(GTK_TEXT_BUFFER(tbuffer_minfo), &end);
  gtk_text_buffer_insert_with_tags_by_name(GTK_TEXT_BUFFER(tbuffer_minfo), &end, name,  strlen(name),  "mt_bold",   NULL);
  gtk_text_buffer_insert_with_tags_by_name(GTK_TEXT_BUFFER(tbuffer_minfo), &end, value, strlen(value), "mt_normal", NULL);
}

static GtkWidget *_gui_minfo_create()
{
  GtkWidget *textview;

  tbuffer_minfo = gtk_text_buffer_new(NULL);
  textview = gtk_text_view_new_with_buffer(tbuffer_minfo);
  gtk_widget_set_size_request(GTK_WIDGET(textview), 200, -1);
  gtk_widget_modify_base(GTK_WIDGET(textview), GTK_STATE_NORMAL, &color_bg);

  gtk_text_buffer_create_tag(GTK_TEXT_BUFFER(tbuffer_minfo), "mt_normal",
                             "editable",       FALSE,
                             "foreground-gdk", &color_text,
                             "foreground-set", TRUE,
                             NULL);

  gtk_text_buffer_create_tag(GTK_TEXT_BUFFER(tbuffer_minfo), "mt_bold",
                             "editable",       FALSE,
                             "weight",         PANGO_WEIGHT_BOLD,
                             "weight-set",     TRUE,
                             "foreground-gdk", &color_text,
                             "foreground-set", TRUE,
                             NULL);

  return textview;
}


void _album_set_artist(gchar*);

gboolean _artist_view_select(GtkTreeSelection *selection, GtkTreeModel *model, GtkTreePath *path, gboolean currently_selected, gpointer userdata)
{
  GtkTreeIter iter;
  gchar       *artist;

  if (currently_selected) return TRUE;

  if (gtk_tree_model_get_iter(model, &iter, path)) {
    gtk_tree_model_get(model, &iter, 0, &artist, -1);
    _album_set_artist(artist);
    xsig_album_list(artist);
  }

  return TRUE;
}

static GtkTreeModel *tmodel_artist;
static GtkWidget    *tview_artist;

static GtkWidget *_gui_artist_create()
{
//  GtkWidget        *check_button;
//  GtkWidget        *entry;
//  GtkWidget        *button;
//  GtkWidget        *image;
  GtkWidget        *swindow;
//  GtkWidget        *hbox;
//  GtkWidget        *vbox;
  GtkCellRenderer  *cell;
  GtkTreeSelection *selection;

  tmodel_artist = (GtkTreeModel*)gtk_list_store_new(1, G_TYPE_STRING);

/*  check_button = gtk_check_button_new_with_label("Artist:");

  entry = gtk_entry_new();
  gtk_widget_modify_base(GTK_WIDGET(entry), GTK_STATE_NORMAL, &color_bg);
  gtk_widget_modify_text(GTK_WIDGET(entry), GTK_STATE_NORMAL, &color_text);
  gtk_widget_modify_fg(GTK_WIDGET(entry), GTK_STATE_NORMAL, &color_text);

  button = gtk_button_new();
  image = gtk_image_new_from_stock("gtk-find", GTK_ICON_SIZE_SMALL_TOOLBAR);
  gtk_container_add(GTK_CONTAINER(button), image);
  gtk_widget_set_size_request(GTK_WIDGET(button), 24, 24);

  hbox = gtk_hbox_new(FALSE, 0);
  gtk_box_pack_start(GTK_BOX(hbox), check_button, FALSE, FALSE, 0);
  gtk_box_pack_start(GTK_BOX(hbox), entry,        TRUE,  TRUE,  0);
  gtk_box_pack_start(GTK_BOX(hbox), button,       FALSE, FALSE, 0);*/

  tview_artist = gtk_tree_view_new();
  cell = gtk_cell_renderer_text_new();
  g_object_set(G_OBJECT(cell), "foreground-gdk", &color_text,
                               "foreground-set", TRUE,
                               NULL);
  gtk_tree_view_insert_column_with_attributes(GTK_TREE_VIEW(tview_artist), -1, "Artist", cell, "text", 0, NULL);
  gtk_tree_view_set_model(GTK_TREE_VIEW(tview_artist), tmodel_artist);
//  gtk_tree_view_set_headers_visible(GTK_TREE_VIEW(tview_artist), FALSE);
  gtk_widget_set_size_request(GTK_WIDGET(tview_artist), 250, 120);
  gtk_widget_modify_base(GTK_WIDGET(tview_artist), GTK_STATE_NORMAL, &color_bg);

  swindow = gtk_scrolled_window_new(NULL, NULL);
  gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(swindow), GTK_SHADOW_ETCHED_IN);
  gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(swindow), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
  gtk_container_add(GTK_CONTAINER(swindow), tview_artist);

/*  vbox = gtk_vbox_new(FALSE, 2);
  gtk_box_pack_start(GTK_BOX(vbox), hbox,    FALSE, FALSE, 0);
  gtk_box_pack_start(GTK_BOX(vbox), swindow, TRUE,  TRUE,  0);*/

  xsig_artist_list();

  selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(tview_artist));
  gtk_tree_selection_set_select_function(selection, _artist_view_select, NULL, NULL);

  return swindow;
}

void gui_artist_clear()
{
  if (tmodel_artist!=NULL) gtk_list_store_clear(GTK_LIST_STORE(tmodel_artist));
}

void gui_artist_detach_model()
{
  g_object_ref(tmodel_artist);
  gtk_tree_view_set_model(GTK_TREE_VIEW(tview_artist), NULL);
}

void gui_artist_atach_model()
{
  gtk_tree_view_set_model(GTK_TREE_VIEW(tview_artist), tmodel_artist);
}

void gui_artist_add(gchar *artist)
{
  GtkTreeIter iter;

  gtk_list_store_append(GTK_LIST_STORE(tmodel_artist), &iter);
  gtk_list_store_set(GTK_LIST_STORE(tmodel_artist), &iter,
                     0, artist,
                     -1);
}


gchar _artist[256];

void _album_set_artist(gchar *artist)
{
  strncpy(_artist, artist, 256);
}

gboolean _album_view_select(GtkTreeSelection *selection, GtkTreeModel *model, GtkTreePath *path, gboolean currently_selected, gpointer userdata)
{
  GtkTreeIter iter;
  gchar       *album;
  static int  skip=0;

  if (skip) {
    skip=0;
    return TRUE;
  }
  
  if (currently_selected) {
    skip=1;
    return TRUE;
  }

  if (gtk_tree_model_get_iter(model, &iter, path)) {
    gtk_tree_model_get(model, &iter, 0, &album, -1);
    xsig_songs_list(_artist, album);
  }

  return TRUE;
}

static GtkTreeModel *tmodel_album;
static GtkWidget    *tview_album;

static GtkWidget *_gui_album_create()
{
  GtkWidget        *swindow;
  GtkCellRenderer  *cell;
  GtkTreeSelection *selection;

  tmodel_album = (GtkTreeModel*)gtk_list_store_new(1, G_TYPE_STRING);

  tview_album = gtk_tree_view_new();
  cell = gtk_cell_renderer_text_new();
  g_object_set(G_OBJECT(cell), "foreground-gdk", &color_text,
                               "foreground-set", TRUE,
                               NULL);
  gtk_tree_view_insert_column_with_attributes(GTK_TREE_VIEW(tview_album), -1, "Album", cell, "text", 0, NULL);
  gtk_tree_view_set_model(GTK_TREE_VIEW(tview_album), tmodel_album);
  gtk_widget_set_size_request(GTK_WIDGET(tview_album), 250, 120);
  gtk_widget_modify_base(GTK_WIDGET(tview_album), GTK_STATE_NORMAL, &color_bg);

  swindow = gtk_scrolled_window_new(NULL, NULL);
  gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(swindow), GTK_SHADOW_ETCHED_IN);
  gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(swindow), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
  gtk_container_add(GTK_CONTAINER(swindow), tview_album);

  selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(tview_album));
  gtk_tree_selection_set_select_function(selection, _album_view_select, NULL, NULL);

  return swindow;
}

void gui_album_clear()
{
  if (tmodel_album!=NULL) gtk_list_store_clear(GTK_LIST_STORE(tmodel_album));
}

void gui_album_detach_model()
{
  g_object_ref(tmodel_album);
  gtk_tree_view_set_model(GTK_TREE_VIEW(tview_album), NULL);
}

void gui_album_atach_model()
{
  gtk_tree_view_set_model(GTK_TREE_VIEW(tview_album), tmodel_album);
}

void gui_album_add(gchar *album)
{
  GtkTreeIter iter;

  gtk_list_store_append(GTK_LIST_STORE(tmodel_album), &iter);
  gtk_list_store_set(GTK_LIST_STORE(tmodel_album), &iter,
                     0, album,
                     -1);
}


#define COL_SONGNO 0
#define COL_ARTIST 1
#define COL_TITLE  2
#define COL_ALBUM  3
#define COL_YEAR   4
#define COL_GENRE  5
#define COL_MLIB   6
#define COL_COL    7

static GtkTreeModel *tmodel_songs;
static GtkWidget    *tview_songs;

static void _songs_view_foreach(GtkTreeModel *model, GtkTreePath *path, GtkTreeIter *iter, gpointer userdata)
{
  guint id;

  gtk_tree_model_get(model, iter, COL_MLIB, &id, -1);
  cmd_playlist_add_id(id);
}

static void cb_songs_add(GtkWidget *widget, gpointer userdata)
{
  GtkTreeSelection *selection;

  selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(tview_songs));
  gtk_tree_selection_selected_foreach(selection, _songs_view_foreach, NULL);
}

static GtkWidget *_gui_songs_create()
{
  GtkWidget            *swindow;
  GtkWidget            *button;
  GtkWidget            *image;
  GtkWidget            *label;
  GtkWidget            *_hbox;
  GtkWidget            *hbox;
  GtkWidget            *vbox;
  GtkTreeSelection     *selection;
  GtkCellRenderer      *cell;
  GtkTreeViewColumn    *column;

  tmodel_songs = (GtkTreeModel*)gtk_list_store_new(8,
                                                   G_TYPE_INT,     // song no. in playlist
                                                   G_TYPE_STRING,  // artist
                                                   G_TYPE_STRING,  // title
                                                   G_TYPE_STRING,  // album
                                                   G_TYPE_STRING,  // year
                                                   G_TYPE_STRING,  // genre
                                                   G_TYPE_INT,     // song no. in medialib
                                                   G_TYPE_STRING); // colour of playing song
  
  tview_songs = gtk_tree_view_new();
  
  cell = gtk_cell_renderer_text_new();
  g_object_set(G_OBJECT(cell), "foreground-gdk", &color_text,
                               "foreground-set", TRUE,
                               "weight",         PANGO_WEIGHT_BOLD,
                               "weight-set",     TRUE,
                               NULL);
                               
  column = gtk_tree_view_column_new_with_attributes("Title", cell,
                                                    "text", COL_TITLE,
                                                    "foreground", COL_COL,
                                                    NULL);
                                                    
  gtk_tree_view_column_set_resizable(GTK_TREE_VIEW_COLUMN(column), TRUE);
  gtk_tree_view_insert_column(GTK_TREE_VIEW(tview_songs),
                              GTK_TREE_VIEW_COLUMN(column), -1);

  cell = gtk_cell_renderer_text_new();
  g_object_set(G_OBJECT(cell), "foreground-gdk", &color_text,
                               "foreground-set", TRUE,
                               NULL);

  column = gtk_tree_view_column_new_with_attributes("Artist", cell,
                                                    "text", COL_ARTIST,
                                                    "foreground", COL_COL,
                                                    NULL);
  gtk_tree_view_column_set_resizable(GTK_TREE_VIEW_COLUMN(column), TRUE);
  gtk_tree_view_insert_column(GTK_TREE_VIEW(tview_songs),
                              GTK_TREE_VIEW_COLUMN(column), -1);


  cell = gtk_cell_renderer_text_new();
  g_object_set(G_OBJECT(cell), "foreground-gdk", &color_text,
                               "foreground-set", TRUE,
                               "style",          PANGO_STYLE_ITALIC,
                               "style-set",      TRUE,
                               NULL);

  column = gtk_tree_view_column_new_with_attributes("Album", cell,
                                                    "text", COL_ALBUM,
                                                    "foreground", COL_COL,
                                                    NULL);
  gtk_tree_view_column_set_resizable(GTK_TREE_VIEW_COLUMN(column), TRUE);
  gtk_tree_view_insert_column(GTK_TREE_VIEW(tview_songs),
                              GTK_TREE_VIEW_COLUMN(column), -1);

  cell = gtk_cell_renderer_text_new();
  g_object_set(G_OBJECT(cell), "foreground-gdk", &color_text,
                               "foreground-set", TRUE,
                               NULL);

  column = gtk_tree_view_column_new_with_attributes("Year", cell,
                                                    "text", COL_YEAR,
                                                    "foreground", COL_COL,
                                                    NULL);
  gtk_tree_view_column_set_resizable(GTK_TREE_VIEW_COLUMN(column), TRUE);
  gtk_tree_view_insert_column(GTK_TREE_VIEW(tview_songs),
                              GTK_TREE_VIEW_COLUMN(column), -1);

  cell = gtk_cell_renderer_text_new();
  g_object_set(G_OBJECT(cell), "foreground-gdk", &color_text,
                               "style",          PANGO_STYLE_ITALIC,
                               "style-set",      TRUE,
                               "foreground-set", TRUE,
                               NULL);

  column = gtk_tree_view_column_new_with_attributes("Genre", cell,
                                                    "text", COL_GENRE,
                                                    "foreground", COL_COL,
                                                    NULL);
  gtk_tree_view_column_set_resizable(GTK_TREE_VIEW_COLUMN(column), TRUE);
  gtk_tree_view_insert_column(GTK_TREE_VIEW(tview_songs),
                              GTK_TREE_VIEW_COLUMN(column), -1);


  gtk_tree_view_set_model(GTK_TREE_VIEW(tview_songs), tmodel_songs);
  gtk_widget_set_size_request(GTK_WIDGET(tview_songs), 250, 120);
  gtk_widget_modify_base(GTK_WIDGET(tview_songs), GTK_STATE_NORMAL, &color_bg);

  swindow = gtk_scrolled_window_new(NULL, NULL);
  gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(swindow), GTK_SHADOW_ETCHED_IN);
  gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(swindow), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
  gtk_container_add(GTK_CONTAINER(swindow), tview_songs);

  selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(tview_songs));
  gtk_tree_selection_set_mode(selection, GTK_SELECTION_MULTIPLE);

  button = gtk_button_new();
  image = gtk_image_new_from_stock("gtk-add", GTK_ICON_SIZE_SMALL_TOOLBAR);
  label = gtk_label_new("Add");
  _hbox = gtk_hbox_new(FALSE, 0);
  gtk_box_pack_start(GTK_BOX(_hbox), image, FALSE, FALSE, 0);
  gtk_box_pack_start(GTK_BOX(_hbox), label, FALSE, FALSE, 0);
  hbox = gtk_hbox_new(TRUE, 0);
  gtk_box_pack_start(GTK_BOX(hbox), _hbox, FALSE, FALSE, 0);
  gtk_container_add(GTK_CONTAINER(button), hbox);

  g_signal_connect(G_OBJECT(button), "clicked",
                   G_CALLBACK(cb_songs_add), NULL);

  vbox = gtk_vbox_new(FALSE, 0);
  gtk_box_pack_start(GTK_BOX(vbox), swindow, TRUE,  TRUE,  0);
  gtk_box_pack_start(GTK_BOX(vbox), button,  FALSE, FALSE, 0);

  return vbox;
}

void gui_songs_clear()
{
  if (tmodel_songs!=NULL) gtk_list_store_clear(GTK_LIST_STORE(tmodel_songs));
}

void gui_songs_add(gchar *artist, gchar *title, gchar *album, gchar *year, gchar *genre, gint id)
{
  GtkTreeIter iter;
  gchar       *p;

  p = "Gray";

  gtk_list_store_append(GTK_LIST_STORE(tmodel_songs), &iter);
  gtk_list_store_set(GTK_LIST_STORE(tmodel_songs), &iter,
                     COL_SONGNO, 0,
                     COL_ARTIST, artist,
                     COL_TITLE,  title,
                     COL_ALBUM,  album,
                     COL_YEAR,   year,
                     COL_GENRE,  genre,
                     COL_MLIB,   id,
                     COL_COL,    p,
                     -1);
}


static GtkTreeModel *tmodel_playlist=NULL;
static guint        playlist_n=1;
static guint        playlist_pos=-1;

static void cb_playlist_activate(GtkTreeView       *tree_view,
                                 GtkTreePath       *path,
                                 GtkTreeViewColumn *col,
                                 gpointer          userdata)
{
  GtkTreeIter iter;
  gint        pno;

  if (gtk_tree_model_get_iter(tmodel_playlist, &iter, path)) {
    gtk_tree_model_get(tmodel_playlist, &iter, 0, &pno, -1);
    cmd_play_pos(pno-1);
  }
}

static GtkWidget *_gui_playlist_create()
{
  GtkWidget         *tree_view;
  GtkWidget         *sw_playlist;
  GtkWidget         *button;
  GtkWidget         *image;
  GtkWidget         *label;
  GtkWidget         *_hbox;
  GtkWidget         *hbox;
  GtkWidget         *vbox;
  GtkCellRenderer   *cell;
  GtkTreeViewColumn *column;

  tmodel_playlist = (GtkTreeModel*)gtk_list_store_new(8,
                                                      G_TYPE_INT,     // song no. in playlist
                                                      G_TYPE_STRING,  // artist
                                                      G_TYPE_STRING,  // title
                                                      G_TYPE_STRING,  // album
                                                      G_TYPE_STRING,  // year
                                                      G_TYPE_STRING,  // genre
                                                      G_TYPE_INT,     // song no. in medialib
                                                      G_TYPE_STRING); // colour of playing song
  
  tree_view = gtk_tree_view_new();
  
  cell = gtk_cell_renderer_text_new();
  g_object_set(G_OBJECT(cell), "foreground-gdk", &color_text,
                               "foreground-set", TRUE,
                               NULL);
  gtk_tree_view_insert_column_with_attributes(GTK_TREE_VIEW(tree_view), -1, NULL, cell,
                                              "text",       COL_SONGNO,
                                              "foreground", COL_COL,
                                              NULL);
  
  cell = gtk_cell_renderer_text_new();
  g_object_set(G_OBJECT(cell), "foreground-gdk", &color_text,
                               "foreground-set", TRUE,
                               "weight",         PANGO_WEIGHT_BOLD,
                               "weight-set",     TRUE,
                               NULL);
  cell = gtk_cell_renderer_text_new();

  column = gtk_tree_view_column_new_with_attributes("Title", cell,
                                                    "text", COL_TITLE,
                                                    "foreground", COL_COL,
                                                    NULL);
  gtk_tree_view_column_set_resizable(GTK_TREE_VIEW_COLUMN(column), TRUE);

  gtk_tree_view_insert_column(GTK_TREE_VIEW(tree_view),
                              GTK_TREE_VIEW_COLUMN(column), -1);

  g_object_set(G_OBJECT(cell), "foreground-gdk", &color_text,
                               "foreground-set", TRUE,
                               NULL);

  column = gtk_tree_view_column_new_with_attributes("Artist", cell,
                                                    "text", COL_ARTIST,
                                                    "foreground", COL_COL,
                                                    NULL);
  gtk_tree_view_column_set_resizable(GTK_TREE_VIEW_COLUMN(column), TRUE);

  gtk_tree_view_insert_column(GTK_TREE_VIEW(tree_view),
                              GTK_TREE_VIEW_COLUMN(column), -1);

/*
  gtk_tree_view_insert_column_with_attributes(GTK_TREE_VIEW(tree_view), -1, "Artist", cell,
                                              "text",       COL_ARTIST,
                                              "foreground", COL_COL,
                                              NULL);
*/

  cell = gtk_cell_renderer_text_new();
  g_object_set(G_OBJECT(cell), "foreground-gdk", &color_text,
                               "foreground-set", TRUE,
                               "style",          PANGO_STYLE_ITALIC,
                               "style-set",      TRUE,
                               NULL);
  
  column = gtk_tree_view_column_new_with_attributes("Album", cell,
                                                    "text", COL_ALBUM,
                                                    "foreground", COL_COL,
                                                    NULL);
  gtk_tree_view_column_set_resizable(GTK_TREE_VIEW_COLUMN(column), TRUE);

  gtk_tree_view_insert_column(GTK_TREE_VIEW(tree_view),
                              GTK_TREE_VIEW_COLUMN(column), -1);

/*
  gtk_tree_view_insert_column_with_attributes(GTK_TREE_VIEW(tree_view), -1, "Album", cell,
                                              "markup",     COL_ALBUM,
                                              "foreground", COL_COL,
                                              NULL);
*/

  cell = gtk_cell_renderer_text_new();
  g_object_set(G_OBJECT(cell), "foreground-gdk", &color_text,
                               "foreground-set", TRUE,
                               NULL);


  column = gtk_tree_view_column_new_with_attributes("Year", cell,
                                                    "text", COL_YEAR,
                                                    "foreground", COL_COL,
                                                    NULL);
  gtk_tree_view_column_set_resizable(GTK_TREE_VIEW_COLUMN(column), TRUE);

  gtk_tree_view_insert_column(GTK_TREE_VIEW(tree_view),
                              GTK_TREE_VIEW_COLUMN(column), -1);

/*
  gtk_tree_view_insert_column_with_attributes(GTK_TREE_VIEW(tree_view), -1, "Year", cell,
                                              "text",       COL_YEAR,
                                              "foreground", COL_COL,
                                              NULL);
*/

  cell = gtk_cell_renderer_text_new();
  g_object_set(G_OBJECT(cell), "foreground-gdk", &color_text,
                               "style",          PANGO_STYLE_ITALIC,
                               "style-set",      TRUE,
                               "foreground-set", TRUE,
                               NULL);

  column = gtk_tree_view_column_new_with_attributes("Genre", cell,
                                                    "text", COL_GENRE,
                                                    "foreground", COL_COL,
                                                    NULL);
  gtk_tree_view_column_set_resizable(GTK_TREE_VIEW_COLUMN(column), TRUE);

  gtk_tree_view_insert_column(GTK_TREE_VIEW(tree_view),
                              GTK_TREE_VIEW_COLUMN(column), -1);
/*
  gtk_tree_view_insert_column_with_attributes(GTK_TREE_VIEW(tree_view), -1, "Genre", cell,
                                              "text",       COL_GENRE,
                                              "foreground", COL_COL,
                                              NULL);
*/

/*  cell = gtk_cell_renderer_text_new();
  g_object_set(G_OBJECT(cell), "foreground-gdk", &color_text,
                               "foreground-set", TRUE,
                               NULL);
  gtk_tree_view_insert_column_with_attributes(GTK_TREE_VIEW(tree_view), -1, "Mlib", cell, "text", 6, NULL);*/

  gtk_tree_view_set_model(GTK_TREE_VIEW(tree_view), tmodel_playlist);

  g_signal_connect(G_OBJECT(tree_view), "row-activated",
                   G_CALLBACK(cb_playlist_activate), NULL);
  
  gtk_widget_set_size_request(GTK_WIDGET(tree_view), 250, 120);
  gtk_widget_modify_base(GTK_WIDGET(tree_view), GTK_STATE_NORMAL, &color_bg);

  sw_playlist = gtk_scrolled_window_new(NULL, NULL);
  gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(sw_playlist), GTK_SHADOW_ETCHED_IN);
  gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(sw_playlist), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
  gtk_container_add(GTK_CONTAINER(sw_playlist), tree_view);

  button = gtk_button_new();
  image = gtk_image_new_from_stock("gtk-clear", GTK_ICON_SIZE_SMALL_TOOLBAR);
  label = gtk_label_new("Clear");
  _hbox = gtk_hbox_new(FALSE, 0);
  gtk_box_pack_start(GTK_BOX(_hbox), image, FALSE, FALSE, 0);
  gtk_box_pack_start(GTK_BOX(_hbox), label, FALSE, FALSE, 0);
  hbox = gtk_hbox_new(TRUE, 0);
  gtk_box_pack_start(GTK_BOX(hbox), _hbox, FALSE, FALSE, 0);
  gtk_container_add(GTK_CONTAINER(button), hbox);

  g_signal_connect(G_OBJECT(button), "clicked",
                   G_CALLBACK(cmd_playlist_clear), NULL);

  vbox = gtk_vbox_new(FALSE, 0);
  gtk_box_pack_start(GTK_BOX(vbox), sw_playlist, TRUE,  TRUE,  0);
  gtk_box_pack_start(GTK_BOX(vbox), button,      FALSE, FALSE, 0);

  return vbox;
}

void gui_playlist_clear()
{
  if (tmodel_playlist!=NULL) gtk_list_store_clear(GTK_LIST_STORE(tmodel_playlist));
  playlist_n=1;
}

void gui_playlist_add(gchar *artist, gchar *title, gchar *album, gchar *year, gchar *genre, gint id)
{
  GtkTreeIter iter;
  gchar       *p;

  if (playlist_n>0 && playlist_pos>=0 && (playlist_pos == playlist_n-1)) p = "White";
  else                                                                   p = "Gray";

  gtk_list_store_append(GTK_LIST_STORE(tmodel_playlist), &iter);
  gtk_list_store_set(GTK_LIST_STORE(tmodel_playlist), &iter,
                     COL_SONGNO, playlist_n,
                     COL_ARTIST, artist,
                     COL_TITLE,  title,
                     COL_ALBUM,  album,
                     COL_YEAR,   year,
                     COL_GENRE,  genre,
                     COL_MLIB,   id,
                     COL_COL,    p,
                     -1);
  playlist_n++;
}

void gui_playlist_set_playing(gint n)
{
  GtkTreeIter iter;

  if ((gint)playlist_pos>=0) if (gtk_tree_model_iter_nth_child(GTK_TREE_MODEL(tmodel_playlist), &iter, NULL, playlist_pos)) {
    gtk_list_store_set(GTK_LIST_STORE(tmodel_playlist), &iter,
                       COL_COL, "Gray",
                       -1);
  }
  
  if (!gtk_tree_model_iter_nth_child(GTK_TREE_MODEL(tmodel_playlist), &iter, NULL, n)) return;
  
  gtk_list_store_set(GTK_LIST_STORE(tmodel_playlist), &iter,
                     COL_COL, "White",
                     -1);
  playlist_pos = n;
}

#undef COL_SONGNO
#undef COL_ARTIST
#undef COL_TITLE
#undef COL_ALBUM
#undef COL_YEAR
#undef COL_GENRE
#undef COL_MLIB
#undef COL_COL


static gboolean cb_delete_event(GtkWidget *widget, GdkEvent *event, gpointer data)
{
  gtk_main_quit();
  xmmsc_disconnect_callback_set(connection, NULL, NULL);
}

static void cb_destroy(GtkWidget *widget, gpointer data)
{
  gtk_main_quit();
  xmmsc_disconnect_callback_set(connection, NULL, NULL);
}

static void cb_window_state(GtkWidget *widget, GdkEventWindowState *event, gpointer userdata)
{
  if (event->new_window_state & GDK_WINDOW_STATE_MAXIMIZED) maximized = 1;
  else                                                      maximized = 0;
}

void gui_window_title(gchar *title)
{
  window_title[window_title_len] = 0;
  strcat(window_title, title);
  gtk_window_set_title(GTK_WINDOW(window), window_title);
}

void gui_window_title_clear()
{
  window_title[window_title_len] = 0;
  gtk_window_set_title(GTK_WINDOW(window), window_title);
}

GtkWidget *gui_main_window_create()
{
  GtkWidget *vbox_controls;
  GtkWidget *separator1;
  GtkWidget *vbox_volume;
  GtkWidget *tview_minfo;
  GtkWidget *vbox_artist;
  GtkWidget *vbox_album;
  GtkWidget *vbox_songs;
  GtkWidget *vbox_playlist;

  GtkWidget *hbox_main;
  GtkWidget *hbox_mlib1;
  GtkWidget *vpaned_mlib;
  GtkWidget *vbox;

  gint      x, y;
  

  window = gtk_window_new(GTK_WINDOW_TOPLEVEL);

  strcpy(window_title, "Azrael " AZ_VERSION);
  window_title_len = strlen(window_title);
  gtk_window_set_title(GTK_WINDOW(window), window_title);

  g_signal_connect(G_OBJECT(window), "delete-event",
                   G_CALLBACK(cb_delete_event), NULL);
  g_signal_connect(G_OBJECT(window), "destroy-event",
                   G_CALLBACK(cb_destroy), NULL);
  g_signal_connect(G_OBJECT(window), "window-state-event",
                   G_CALLBACK(cb_window_state), NULL);

  color_bg.red     = 0x0000;
  color_bg.green   = 0x0000;
  color_bg.blue    = 0x0000;

  color_text.pixel = 0;
  color_text.red   = 0xBFFF;
  color_text.green = 0xBFFF;
  color_text.blue  = 0xBFFF;

  gtk_container_set_border_width(GTK_CONTAINER(window), 6);


  vbox_controls       = _gui_controls_create();
  separator1          = gtk_vseparator_new();
  vbox_volume         = _gui_volume_create();
  _gui_separator2     = gtk_vseparator_new();
  _gui_vbox_equalizer = _gui_equalizer_create();
  tview_minfo         = _gui_minfo_create();
  vbox_artist         = _gui_artist_create();
  vbox_album          = _gui_album_create();
  vbox_songs          = _gui_songs_create();
  vbox_playlist       = _gui_playlist_create();

  hbox_main = gtk_hbox_new(FALSE, 6);
  gtk_box_pack_start(GTK_BOX(hbox_main), vbox_controls,       FALSE, FALSE, 0);
  gtk_box_pack_start(GTK_BOX(hbox_main), separator1,          FALSE, FALSE, 0);
  gtk_box_pack_start(GTK_BOX(hbox_main), vbox_volume,         FALSE, FALSE, 0);
  gtk_box_pack_start(GTK_BOX(hbox_main), _gui_separator2,     FALSE, FALSE, 0);
  gtk_box_pack_start(GTK_BOX(hbox_main), _gui_vbox_equalizer, FALSE, FALSE, 0);
  gtk_box_pack_start(GTK_BOX(hbox_main), tview_minfo,         TRUE,  TRUE,  0);
  gtk_widget_get_size_request(hbox_main, &x, &y);

  hbox_mlib1 = gtk_hbox_new(FALSE, 6);
  gtk_box_pack_start(GTK_BOX(hbox_mlib1), vbox_artist, TRUE, TRUE, 0);
  gtk_box_pack_start(GTK_BOX(hbox_mlib1), vbox_album,  TRUE, TRUE, 0);

  vpaned_mlib = gtk_vpaned_new();
  gtk_paned_pack1(GTK_PANED(vpaned_mlib), hbox_mlib1, FALSE, FALSE);
  gtk_paned_pack2(GTK_PANED(vpaned_mlib), vbox_songs, TRUE,  FALSE);

  _gui_notebook = gtk_notebook_new();
  gtk_notebook_set_tab_pos(GTK_NOTEBOOK(_gui_notebook), GTK_POS_TOP);
  gtk_notebook_append_page(GTK_NOTEBOOK(_gui_notebook), vbox_playlist, gtk_label_new("Playlist"));
  gtk_notebook_append_page(GTK_NOTEBOOK(_gui_notebook), vpaned_mlib, gtk_label_new("MediaLib"));

  vbox = gtk_vbox_new(FALSE, 6);
  gtk_box_pack_start(GTK_BOX(vbox), hbox_main, FALSE, FALSE, 0);
  gtk_box_pack_start(GTK_BOX(vbox), _gui_notebook,  TRUE,  TRUE,  0);

  gtk_container_add(GTK_CONTAINER(window), vbox);

  hints.max_height = y;
  hints.max_width  = 1024;

  return window;
}

