/*   azrael - client for the xmms2 player
 *   Copyright (C) 2005  Costin Chirvasuta
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "az_lcd.h"

#define LCD_WIDTH  101
#define LCD_HEIGHT  43

GtkWidget *darea = NULL;

guint    lcd_min, lcd_sec;
guint    lcd_value;
guint    lcd_total=-1;
gint     leds[4][7];
gboolean lcd_countdown=FALSE;

void update_leds()
{
  gint i;
  gint digit[4];

  digit[0] = lcd_min/10;
  digit[1] = lcd_min%10;
  digit[2] = lcd_sec/10;
  digit[3] = lcd_sec%10;

  for (i=0; i<4; i++) {
    switch (digit[i]) {
      case 0: leds[i][0] = 1; leds[i][1] = 1; leds[i][2] = 1; leds[i][3] = 0; leds[i][4] = 1; leds[i][5] = 1; leds[i][6] = 1; break;
      case 1: leds[i][0] = 0; leds[i][1] = 0; leds[i][2] = 1; leds[i][3] = 0; leds[i][4] = 0; leds[i][5] = 1; leds[i][6] = 0; break;
      case 2: leds[i][0] = 1; leds[i][1] = 0; leds[i][2] = 1; leds[i][3] = 1; leds[i][4] = 1; leds[i][5] = 0; leds[i][6] = 1; break;
      case 3: leds[i][0] = 1; leds[i][1] = 0; leds[i][2] = 1; leds[i][3] = 1; leds[i][4] = 0; leds[i][5] = 1; leds[i][6] = 1; break;
      case 4: leds[i][0] = 0; leds[i][1] = 1; leds[i][2] = 1; leds[i][3] = 1; leds[i][4] = 0; leds[i][5] = 1; leds[i][6] = 0; break;
      case 5: leds[i][0] = 1; leds[i][1] = 1; leds[i][2] = 0; leds[i][3] = 1; leds[i][4] = 0; leds[i][5] = 1; leds[i][6] = 1; break;
      case 6: leds[i][0] = 1; leds[i][1] = 1; leds[i][2] = 0; leds[i][3] = 1; leds[i][4] = 1; leds[i][5] = 1; leds[i][6] = 1; break;
      case 7: leds[i][0] = 1; leds[i][1] = 0; leds[i][2] = 1; leds[i][3] = 0; leds[i][4] = 0; leds[i][5] = 1; leds[i][6] = 0; break;
      case 8: leds[i][0] = 1; leds[i][1] = 1; leds[i][2] = 1; leds[i][3] = 1; leds[i][4] = 1; leds[i][5] = 1; leds[i][6] = 1; break;
      case 9: leds[i][0] = 1; leds[i][1] = 1; leds[i][2] = 1; leds[i][3] = 1; leds[i][4] = 0; leds[i][5] = 1; leds[i][6] = 1; break;
    }
  }
}

void lcd_set_countdown(gboolean b)
{
  gboolean ret;
  guint    newval;

  if (b && lcd_total!=-1) {
    lcd_countdown=TRUE;
    lcd_set_value(lcd_value);
  }
  else {
    lcd_countdown=FALSE;
    newval = (lcd_total>=lcd_value)?(lcd_total-lcd_value):0;
    lcd_set_value(newval);
  }

  
  if (darea && darea->window) g_signal_emit_by_name(G_OBJECT(darea), "expose-event", darea, &ret);
}

void lcd_set_total(guint total)
{
  gboolean ret;

  lcd_total = total%3600;
  if (darea && darea->window) g_signal_emit_by_name(G_OBJECT(darea), "expose-event", darea, &ret);
}

void lcd_set_value(guint val)
{
  gboolean ret;

  lcd_value = val%3600;
  if (lcd_countdown) lcd_value = (lcd_total>=lcd_value)?(lcd_total-lcd_value):0;
  
  lcd_min   = lcd_value/60;
  lcd_sec   = lcd_value%60;

  update_leds();
  if (darea && darea->window) g_signal_emit_by_name(G_OBJECT(darea), "expose-event", darea, &ret);
}

void lcd_set_min_sec(guint min, guint sec)
{
  gboolean ret;

  lcd_min   = min%60;
  lcd_sec   = sec%60;
  lcd_value = lcd_min*60+lcd_sec;

  update_leds();
  if (darea && darea->window) g_signal_emit_by_name(G_OBJECT(darea), "expose-event", darea, &ret);
}

gboolean on_darea_expose(GtkWidget      *widget,
                         GdkEventExpose *event,
                         gpointer       userdata)
{
  GdkGC    *gc1, *gc2;
  GdkColor color;
  gint     pad, i;

  gc1 = gdk_gc_new(widget->window);
  gc2 = gdk_gc_new(widget->window);

  color.red   = 0xAFFF;
  color.green = 0xAFFF;
  color.blue  = 0xAFFF;
  gdk_gc_set_rgb_fg_color(gc1, &color);

  color.red   = 0x0000;
  color.green = 0x0000;
  color.blue  = 0x0000;
  gdk_gc_set_rgb_fg_color(gc2, &color);

  gdk_draw_rectangle(widget->window, gc2, TRUE, 0, 0, LCD_WIDTH, LCD_HEIGHT);

#define HPAD 11
#define VPAD 5
#define HWID 1

  if (lcd_countdown) gdk_draw_rectangle(widget->window, gc1, TRUE, HPAD-7, VPAD+15, 4, 3);

  gdk_draw_rectangle(widget->window, gc1, TRUE, HPAD+(HWID+19)*2, VPAD+10, 3, 3);
  gdk_draw_rectangle(widget->window, gc1, TRUE, HPAD+(HWID+19)*2, VPAD+21, 3, 3);

  for (i=0; i<4; i++) {
    switch (i) {
      case 0: pad = HPAD +           0; break;
      case 1: pad = HPAD + 19 + HWID*1; break;
      case 2: pad = HPAD + 44 + HWID*2; break;
      case 3: pad = HPAD + 63 + HWID*3; break;
    }

    if (leds[i][0]) {
      gdk_draw_line(widget->window, gc1, pad +  3, VPAD +  0, pad + HWID + 12, VPAD +  0);
      gdk_draw_line(widget->window, gc1, pad +  2, VPAD +  1, pad + HWID + 13, VPAD +  1);
      gdk_draw_line(widget->window, gc1, pad +  3, VPAD +  2, pad + HWID + 12, VPAD +  2);
    }

    if (leds[i][1]) {
      gdk_draw_line(widget->window, gc1, pad +  0, VPAD +  4, pad +  0, VPAD + 13);
      gdk_draw_line(widget->window, gc1, pad +  1, VPAD +  3, pad +  1, VPAD + 14);
      gdk_draw_line(widget->window, gc1, pad +  2, VPAD +  4, pad +  2, VPAD + 13);
    }

    if (leds[i][2]) {
      gdk_draw_line(widget->window, gc1, pad + HWID + 13, VPAD +  4, pad + HWID + 13, VPAD + 13);
      gdk_draw_line(widget->window, gc1, pad + HWID + 14, VPAD +  3, pad + HWID + 14, VPAD + 14);
      gdk_draw_line(widget->window, gc1, pad + HWID + 15, VPAD +  4, pad + HWID + 15, VPAD + 13);
    }

    if (leds[i][3]) {
      gdk_draw_line(widget->window, gc1, pad +  3, VPAD + 15, pad + HWID + 12, VPAD + 15);
      gdk_draw_line(widget->window, gc1, pad +  2, VPAD + 16, pad + HWID + 13, VPAD + 16);
      gdk_draw_line(widget->window, gc1, pad +  3, VPAD + 17, pad + HWID + 12, VPAD + 17);
    }

    if (leds[i][4]) {
      gdk_draw_line(widget->window, gc1, pad +  0, VPAD + 19, pad +  0, VPAD + 28);
      gdk_draw_line(widget->window, gc1, pad +  1, VPAD + 18, pad +  1, VPAD + 29);
      gdk_draw_line(widget->window, gc1, pad +  2, VPAD + 19, pad +  2, VPAD + 28);
    }

    if (leds[i][5]) {
      gdk_draw_line(widget->window, gc1, pad + HWID + 13, VPAD + 19, pad + HWID + 13, VPAD + 28);
      gdk_draw_line(widget->window, gc1, pad + HWID + 14, VPAD + 18, pad + HWID + 14, VPAD + 29);
      gdk_draw_line(widget->window, gc1, pad + HWID + 15, VPAD + 19, pad + HWID + 15, VPAD + 28);
    }

    if (leds[i][6]) {
      gdk_draw_line(widget->window, gc1, pad +  3, VPAD + 30, pad + HWID + 12, VPAD + 30);
      gdk_draw_line(widget->window, gc1, pad +  2, VPAD + 31, pad + HWID + 13, VPAD + 31);
      gdk_draw_line(widget->window, gc1, pad +  3, VPAD + 32, pad + HWID + 12, VPAD + 32);
    }
  }

#undef HPAD
#undef VPAD
#undef HWID
}

GtkWidget *lcd_create()
{
  darea = gtk_drawing_area_new();
  gtk_drawing_area_size(GTK_DRAWING_AREA(darea), LCD_WIDTH, LCD_HEIGHT);
  g_signal_connect(G_OBJECT(darea), "expose-event",
                   G_CALLBACK(on_darea_expose), NULL);

  lcd_set_value(0);

  return darea;
}

