/*   azrael - client for the xmms2 player
 *   Copyright (C) 2005  Costin Chirvasuta
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "azrael.h"
#include "az_lcd.h"
#include "az_cmd.h"
#include "az_xsig.h"
#include "az_gui.h"

guint ticks;

static gboolean delete_event(GtkWidget *widget, GdkEvent *event, gpointer connection)
{
  gtk_main_quit();
  xmmsc_disconnect_callback_set((xmmsc_connection_t*)connection, NULL, NULL);
}

static void destroy(GtkWidget *widget, gpointer connection)
{
  gtk_main_quit();
  xmmsc_disconnect_callback_set((xmmsc_connection_t*)connection, NULL, NULL);
}

void az_set_ticks(guint t)
{
  ticks=t;
}

gboolean timeout_func(gpointer data)
{
  if (!ticks) return TRUE;

  ticks--;

  if (ticks-6==0) {
    ticks=0;
    xsig_playlist_list_imm();
    return TRUE;
  }
  
  if (!ticks) {
    gui_finfo_stat_show();
  }

  return TRUE;
}

int main(int argc, char **argv)
{
  GtkWidget *window;

  gchar     *path, sz_error[256];

  guint     i;
  guint     ivol;
  gint      to_tag;


  gtk_init(&argc, &argv);

  connection = xmmsc_init("azrael");

  path = getenv("XMMS_PATH");
  if (!xmmsc_connect(connection, path)) {
    sprintf(sz_error, "Could not connect to xmms2d: %s\n", xmmsc_get_last_error(connection));
    gui_error(sz_error);
    return 1;
  }
  
  window = gui_main_window_create();

  xmmsc_mainloop_gmain_init(connection);

  XMMS_CALLBACK_SET(connection, xmmsc_broadcast_configval_changed, xsig_config, NULL);
  XMMS_CALLBACK_SET(connection, xmmsc_signal_playback_playtime, xsig_playtime, NULL);
  XMMS_CALLBACK_SET(connection, xmmsc_broadcast_playback_status, xsig_status, NULL);
  XMMS_CALLBACK_SET(connection, xmmsc_playback_status, xsig_status, NULL);
  XMMS_CALLBACK_SET(connection, xmmsc_broadcast_playback_current_id, xsig_media, connection);
  XMMS_CALLBACK_SET(connection, xmmsc_broadcast_medialib_entry_changed, xsig_media, connection);
  XMMS_CALLBACK_SET(connection, xmmsc_playback_current_id, xsig_media, connection);
  XMMS_CALLBACK_SET(connection, xmmsc_broadcast_playlist_current_pos, xsig_playlist_pos, NULL);
  XMMS_CALLBACK_SET(connection, xmmsc_broadcast_playlist_changed, xsig_playlist_list, NULL);

  gtk_widget_show_all(window);

  az_set_ticks(0);
  to_tag = g_timeout_add(1000, timeout_func, NULL);

  xsig_playlist_list();

  gtk_main();

  g_source_remove(to_tag);
  if (connection) xmmsc_unref(connection);
  
  return 0;
}

