/*   azrael - client for the xmms2 player
 *   Copyright (C) 2005  Costin Chirvasuta
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "az_xsig.h"

void print_result_type(xmmsc_result_t *res)
{
  int type;

  type = xmmsc_result_get_type(res);

//  printf("RES: %d\n", type);
  switch (type) {
    case XMMSC_RESULT_VALUE_TYPE_NONE:   printf("RES: TYPE_NONE\n");          break;
    case XMMSC_RESULT_VALUE_TYPE_UINT32: printf("RES: TYPE_UINT32\n");        break;
    case XMMSC_RESULT_VALUE_TYPE_INT32:  printf("RES: TYPE_INT32\n");         break;
    case XMMSC_RESULT_VALUE_TYPE_STRING: printf("RES: TYPE_STRING\n");        break;
    default:                             printf("RES: UNKNOWN (%d)\n", type); break;
  }
}

void print_dict_type(xmmsc_result_t *res, gchar *key)
{
  xmmsc_result_value_type_t type;

  type = xmmsc_result_get_dict_entry_type(res, key);

  switch (type) {
    case XMMSC_RESULT_VALUE_TYPE_NONE:   printf("DICT: \"%s\" - TYPE_NONE\n",   key); break;
    case XMMSC_RESULT_VALUE_TYPE_UINT32: printf("DICT: \"%s\" - TYPE_UINT32\n", key); break;
    case XMMSC_RESULT_VALUE_TYPE_INT32:  printf("DICT: \"%s\" - TYPE_INT32\n",  key); break;
    case XMMSC_RESULT_VALUE_TYPE_STRING: printf("DICT: \"%s\" - TYPE_STRING\n", key); break;
  }
}

void xsig_config(xmmsc_result_t *res, void *userdata)
{
  xmmsc_result_t *newres;
  gint           r32;
  gchar          *str;

  if (xmmsc_result_iserror(res)) printf("result is error\n");
  if (xmmsc_result_is_list(res)) printf("result is list\n");

  print_result_type(res);
  if (xmmsc_result_get_int(res, &r32)) printf("%d\n", r32);
}

void handle_media(xmmsc_result_t*, void*);

void xsig_playtime(xmmsc_result_t *res, void *userdata)
{
  guint          time;
  static guint   lasttime = -1;
  xmmsc_result_t *newres;

  if (!xmmsc_result_iserror(res) && xmmsc_result_get_uint(res, &time)) {}
  else return;

  newres = xmmsc_result_restart(res);
  xmmsc_result_unref(res);
  xmmsc_result_unref(newres);

  time /= 1000;
  if (time != lasttime) {
    lasttime = time;
    lcd_set_value(time);
    gui_prog_ignore(TRUE);
    gui_prog_set(time);
  }
}

void xsig_status(xmmsc_result_t *res, void *userdata)
{
  guint status;
  gchar strstat[256];

  if (!xmmsc_result_get_uint(res, &status)) return;

  switch (status) {
    case XMMS_PLAYBACK_STATUS_PLAY:
         gui_window_title_clear();
         break;
    case XMMS_PLAYBACK_STATUS_PAUSE:
         gui_window_title(" - PAUSED");
         break;
    case XMMS_PLAYBACK_STATUS_STOP:
         lcd_set_total(0);
         lcd_set_value(0);

         gui_prog_ignore(TRUE);
         gui_prog_clear();

         gui_minfo_clear();
         gui_minfo("\n  Artist: ", " \n");
         gui_minfo("  Title: ", " \n");
         gui_minfo("  Album: ", " \n");
         gui_minfo("  Year: ", " \n");
         gui_minfo("  Genre: ", " \n");

         sprintf(strstat, "0:00   0 Hz   0 Kbps");
         gui_finfo_stat(strstat);
         gui_finfo_stat_show();
    
         gui_window_title(" - STOPPED");
         break;
  }
}

void xsig_media(xmmsc_result_t *res, void *connection)
{
  xmmsc_result_t *newres;

  if (!xmmsc_result_get_uint(res, &global_id)) return;

  newres = xmmsc_medialib_get_info(connection, global_id);
  xmmsc_result_notifier_set(newres, handle_media, connection);
  xmmsc_result_unref(newres);
}

void handle_media(xmmsc_result_t *res, void *connection)
{
  gchar   str[512], *ret;
  guint   duration, samplerate, bitrate;

  res = xmmsc_medialib_get_info(connection, global_id);
  xmmsc_result_wait(res);

  gui_minfo_clear();

  xmmsc_result_get_dict_entry_str(res, "artist", &ret);
  snprintf(str, 511, "%s\n", (ret)?ret:" ");
  gui_minfo("\n  Artist: ", str);

  xmmsc_result_get_dict_entry_str(res, "title", &ret);
  snprintf(str, 511, "%s\n", (ret)?ret:" ");
  gui_minfo("  Title: ", str);

  xmmsc_result_get_dict_entry_str(res, "album", &ret);
  snprintf(str, 511, "%s\n", (ret)?ret:" ");
  gui_minfo("  Album: ", str);

  xmmsc_result_get_dict_entry_str(res, "date", &ret);
  snprintf(str, 511, "%s\n", (ret)?ret:" ");
  gui_minfo("  Year: ", str);

  xmmsc_result_get_dict_entry_str(res, "genre", &ret);
  snprintf(str, 511, "%s\n", (ret)?ret:" ");
  gui_minfo("  Genre: ", str);

  if (xmmsc_result_get_dict_entry_int32(res, "duration", &duration)) {}
  else duration = 0;

  if (xmmsc_result_get_dict_entry_int32(res, "samplerate", &samplerate)) {}
  else samplerate = 0;

  if (xmmsc_result_get_dict_entry_int32(res, "bitrate", &bitrate)) {}
  else bitrate = 0;

  duration /= 1000;
  gui_prog_ignore(TRUE);
  gui_prog_upper(duration);
  gui_prog_set(0);
  lcd_set_total(duration);

  bitrate /= 1000;

  snprintf(str, 256, "%d:%02d   %d Hz   %d Kbps", duration/60, duration%60, (!!bitrate)*44100, bitrate);
  gui_finfo_stat(str);
  gui_finfo_stat_show();

  xmmsc_result_unref(res);
}

void _song_info(xmmsc_result_t *res2, void *userdata)
{
  gint32              id;
  gchar               *url, *filename = "", *artist_tmp = "", *title_tmp = "";
  gchar               *album, *artist = NULL, *title = NULL, *channel = NULL;
  gchar               *year, *genre;
  gchar               *prefix = "";
  gchar               album_with_tracknr[256];
  gint                tracknr = -1;
  guint               playlist_pos;
  gint                target;

  xmmsc_result_get_dict_entry_str(res2,     "artist",  &artist_tmp);
  xmmsc_result_get_dict_entry_str(res2,     "title",   &title_tmp);
  xmmsc_result_get_dict_entry_str(res2,     "album",   &album);
  xmmsc_result_get_dict_entry_int32(res2,   "id",      &id);
  xmmsc_result_get_dict_entry_str(res2,     "url",     &url);
  xmmsc_result_get_dict_entry_int32(res2,   "tracknr", &tracknr);
  xmmsc_result_get_dict_entry_str(res2,     "date",    &year);
  xmmsc_result_get_dict_entry_str(res2,     "genre",   &genre);

  if (xmmsc_result_get_dict_entry_str(res2, "channel", &channel)) {
    prefix = "[Stream] ";
    album = channel;
  } else if (album == NULL)
    album = "[Unknown Album]";

  if (tracknr > 0) {
    snprintf(album_with_tracknr, 256, "%s [<b>%02d</b>]", album, tracknr);
  } else {
    snprintf(album_with_tracknr, 256, "%s", album);
  }

  if (url) filename = g_path_get_basename(url);
  else     filename = "[Unknown]";

  if (artist_tmp == NULL && channel == NULL) {
    artist = g_malloc(sizeof(gchar) * 10);
    g_strlcpy(artist, "[Unknown]", sizeof(gchar) * 10);
  } else if (artist_tmp != NULL) {
    artist = g_malloc(sizeof(gchar) * (strlen(artist_tmp) + strlen(prefix) + 1));
    g_snprintf(artist, strlen(artist_tmp) + strlen(prefix) + 1,
               "%s%s", prefix, artist_tmp);
  } else if (channel != NULL) {
    artist = g_malloc(sizeof(gchar) * (strlen(prefix) + 1));
    g_snprintf(artist, strlen(prefix) + 1,
               "%s", prefix);
  }

  if (title_tmp == NULL) {
    title = g_malloc(sizeof(gchar) * strlen(filename) + 1);
    g_strlcpy(title, filename, sizeof(gchar) * strlen(filename) + 1);
  } else {
    title = g_malloc(sizeof(gchar) * strlen(title_tmp) + 1);
    g_strlcpy(title, title_tmp, sizeof(gchar) * strlen(title_tmp) + 1);
  }

  target = GPOINTER_TO_INT(userdata);
  switch (target) {
    case 1: gui_playlist_add(artist, title, album_with_tracknr, year, genre, id);
            break;
    case 2: gui_songs_add(artist, title, album_with_tracknr, year, genre, id);
            break;
  }

  g_free(artist);
  g_free(title);

  if (url) g_free(filename);

  res2 = xmmsc_playlist_current_pos(connection);
  xmmsc_result_wait(res2);
  xsig_playlist_pos(res2, NULL);
}

void _playlist_list(xmmsc_result_t *res, void *userdata)
{
  xmmsc_result_t *res2;
  guint          i;

  for (; xmmsc_result_list_valid(res); xmmsc_result_list_next(res)) {
    xmmsc_result_get_uint(res, &i);
    res2 = xmmsc_medialib_get_info(connection, i);
    xmmsc_result_notifier_set(res2, _song_info, GINT_TO_POINTER(1));
    xmmsc_result_unref(res2);
  }
}

static int plist_imm=0;

void xsig_playlist_list_imm()
{
  plist_imm=1;
  xsig_playlist_list();
}

void xsig_playlist_list()
{
  xmmsc_result_t *res;

  gui_playlist_clear();
  
  if (!plist_imm) {
    az_set_ticks(9);
    return;
  }

  plist_imm=0;
  
  res = xmmsc_playlist_list(connection);
  xmmsc_result_notifier_set(res, _playlist_list, NULL);
  xmmsc_result_unref(res);
}

void xsig_playlist_pos(xmmsc_result_t *res, void *userdata)
{
  guint n;

  xmmsc_result_get_uint(res, &n);
  gui_playlist_set_playing(n);
}


void _artist_list(xmmsc_result_t *res, void *userdata)
{
  gchar *artist;
  gchar artist_tmp[256] = "";

  gui_artist_clear();
  gui_artist_detach_model();

  for (; xmmsc_result_list_valid(res); xmmsc_result_list_next(res)) {
    xmmsc_result_get_dict_entry_str(res, "value", &artist);
    if (!artist) artist = "Unknown";
    if (strcmp(artist, artist_tmp)) {
      strncpy(artist_tmp, artist, sizeof(artist_tmp));
      gui_artist_add(artist);
    }
  }

  gui_artist_atach_model();
}

void xsig_artist_list()
{
  xmmsc_result_t *res;
  gchar          *query = "SELECT DISTINCT * FROM Media WHERE key='artist'";

  res = xmmsc_medialib_select(connection, query);
  xmmsc_result_notifier_set(res, _artist_list, NULL);
  xmmsc_result_unref(res);
}

void _album_list(xmmsc_result_t *res, void *userdata)
{
  gchar *album;
  gchar album_tmp[256] = "";

  gui_album_clear();
  gui_album_detach_model();

  for (; xmmsc_result_list_valid(res); xmmsc_result_list_next(res)) {
    xmmsc_result_get_dict_entry_str(res, "value", &album);
    if (!album) album = "Unknown";
    if (strcmp(album, album_tmp)) {
      strncpy(album_tmp, album, sizeof(album_tmp));
      gui_album_add(album);
    }
  }

  gui_album_atach_model();
}

void xsig_album_list(char *artist)
{
  xmmsc_result_t *res;
  gchar          query[1024] = "";

  sprintf(query, "SELECT DISTINCT m1.value FROM Media AS m0, Media AS m1 WHERE m0.key = LOWER('artist') "
                 "AND LOWER(m0.value) LIKE LOWER('%s') AND m1.key = LOWER('album') and m0.id = m1.id", artist);
  res = xmmsc_medialib_select(connection, query);
  xmmsc_result_notifier_set(res, _album_list, NULL);
  xmmsc_result_unref(res);
}

void _songs_list(xmmsc_result_t *res, void *userdata)
{
  xmmsc_result_t *res2;
  gint           i;

  gui_songs_clear();

  for (;xmmsc_result_list_valid(res); xmmsc_result_list_next(res)) {
    xmmsc_result_get_dict_entry_int32(res, "id", &i);
    res2 = xmmsc_medialib_get_info(connection, i);
    xmmsc_result_notifier_set(res2, _song_info, GINT_TO_POINTER(2));
    xmmsc_result_unref(res2);
  }
}

void xsig_songs_list(char *artist, char *album)
{
  xmmsc_result_t *res;
  gchar          query[1024] = "";

  sprintf(query, "SELECT DISTINCT m0.id FROM Media AS m0, Media AS m1 WHERE m0.key = LOWER('artist') "
                 "AND LOWER(m0.value) LIKE LOWER('%s') AND m1.key = LOWER('album') AND "
                 "LOWER(m1.value) LIKE LOWER('%s') AND m0.id = m1.id", artist, album);
  res = xmmsc_medialib_select(connection, query);
  xmmsc_result_notifier_set(res, _songs_list, NULL);
  xmmsc_result_unref(res);
}

